import os, sys
from Ft.Rdf import Model
from Ft.Rdf.Serializers import Dom
from Ft.Lib import pDomlette

# helper functions for statement comparision
sys.path.append('..')
import Helpers

DATABASE_NAME = os.environ.get('RDF_TEST_DB', 'test')

SOURCE_URI = ''

source = """<rdf:RDF 
  xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#'
  xmlns:ns='http://my/ns#'>

  <ns:item ID='this'> <!-- typedNode -->
    <ns:prop>  <!-- propertyElt -->
      <ns:item>  <!-- typedNode -->
        <ns:prop resource='http://foo/'/>  <!-- propertyElt -->
      </ns:item> 
    </ns:prop> 
  </ns:item> 
</rdf:RDF>"""

deserialized = [
    ('#this',
     'http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
     'http://my/ns#item',
     '',
     ''),
    ('ANONYMOUS0',
     'http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
     'http://my/ns#item',
     '',
     ''),
    ('ANONYMOUS0',
     'http://my/ns#prop',
     'http://foo/',
     '',
     ''),
    ('#this',
     'http://my/ns#prop',
     'ANONYMOUS0',
     '',
     ''),
    ]

def test(tester):
    tester.startGroup('Borrowed: jk_20010329')
    tester.startTest('Initialize driver [%s]' % tester.rdf_driver_name)
    if tester.rdf_driver.ExistsDb(DATABASE_NAME):
        tester.rdf_driver.DestroyDb(DATABASE_NAME)
    tester.rdf_driver.CreateDb(DATABASE_NAME)
    tester.testDone()
    
    tester.startTest('Test 1')
    db = tester.rdf_driver.GetDb('test')
    db.begin()
    model = Model.Model(db)
    serializer = Dom.Serializer()
    reader = pDomlette.PyExpatReader()
    dom = reader.fromString(source)
    serializer.deserialize(model, dom, SOURCE_URI)
    reader.releaseNode(dom)
    actual = Helpers.make_tuple(model.statements(), SOURCE_URI)
    tester.compare(deserialized, actual, func=Helpers.compare_tuple)
    db.rollback()
    tester.testDone()
    return tester.groupDone()

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(useColor=1)
    tester.rdf_driver_name = 'Memory'
    from Ft.Rdf.Drivers import Memory
    tester.rdf_driver = Memory
    test(tester)
