#Test all pf RDF
import os, time

g_drivers = ['Memory',
             'Dbm',
             'Postgres',
             ]

g_tests = [
    ('Drivers',
     ['test_interface',
      ],
     # ignored
     ['test_dbm',
      'test_postgres',
      'test_memory',
      ],
     ),

    ('',
     ['test_model',
      'test_serializer',
      ],
     # ignored
     ['Helpers',
      'TestSuite',
      'test',
    ],
     ),

    ('borrowed',
     ['jk_20010329',
      ],
     # ignored
     [],
     ),
    ]

def dir_check(tester, tests, ignored):
    import glob
    tester.startTest("Dir Check")
    files = glob.glob('*.py')
    omitted = 0
    for file in files:
        # Just the base filename (no ext)
        mod_name = os.path.basename(file)[:-3]
        if mod_name not in tests and mod_name not in ignored:
            tester.warning('Warning: '+ file + ' is omitted from test list.\n')
            #sys.stderr.write('Warning: '+ file + ' is omitted from test list.\n')
            omitted = 1
    tester.testDone()
    if omitted:
        time.sleep(5)

def test(tester, drivers=None):
    drivers = drivers or g_drivers
    driver_map = {}

    tester.startGroup("Load Drivers")
    for driver in drivers:
        tester.startTest(driver)
        try:
            module = __import__('Ft.Rdf.Drivers.%s' % driver, {}, {}, '*')
            driver_map[driver] = module
        except:
            tester.warning('Driver not loaded')
        tester.testDone()
    tester.groupDone()

    total = 0.0
    curr_dir = os.getcwd()
    for (path, tests, ignored) in g_tests:
        dir = os.path.join(curr_dir, path)
        os.chdir(dir)
        tests.sort()
        tester.startGroup('Dir Check of %s' % dir)
        dir_check(tester, tests, ignored)
        tester.groupDone()

        for test in tests:
            m = __import__(test)
            for (name, driver) in driver_map.items():
                tester.rdf_driver_name = name
                tester.rdf_driver = driver
                start = time.time()
                m.test(tester)
                total = total + time.time() - start

    os.chdir(curr_dir)

    print '-'*50
    tester.report()
    print '-'*50
    print 'Total time for tests (in seconds): %0.2f' % total
    print '-'*50


if __name__ == '__main__':
    import sys
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    test(tester, sys.argv[1:])
