import test_harness
from xml.xslt import XsltException, Error

sheet_str_1 = """<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:n="http://spam.com"
  version="1.0"
>

  <xsl:template match="/">
    <HTML>
    <HEAD><TITLE>Address Book</TITLE>
    </HEAD>
    <BODY>
    <TABLE><xsl:apply-templates/></TABLE>
    </BODY>
    </HTML>
  </xsl:template>

  <xsl:variable name='a'>1</xsl:variable>

  <xsl:variable name='width' select='100'></xsl:variable>

  <xsl:variable name='c'></xsl:variable>

  <xsl:template match="ENTRY">
        <!-- Make sure shadowing check isn't overzealous -->
        <xsl:param name='n:a' select='ok'/>
        <!-- Tests legal shadowing -->
        <xsl:variable name='a'>boo</xsl:variable>
        <xsl:element name='TR'>
        <xsl:apply-templates select='NAME'/>
        </xsl:element>
  </xsl:template>

  <xsl:template match="NAME">
    <xsl:variable name='d' select='.'></xsl:variable>
    <TD WIDTH='{$d}'>
      <B><xsl:apply-templates/></B>
    </TD>
  </xsl:template>

</xsl:stylesheet>
"""


sheet_str_2 = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">

  <xsl:param name='override' select='abc'/>

  <xsl:template match="/">
    <doc>
      <xsl:value-of select='$override'/>
    </doc>
  </xsl:template>

</xsl:stylesheet>
"""


invalid_sheet_str_1 = """\
<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <xsl:variable name='x' select='a'/>

  <xsl:template match="/">
    <xsl:variable name='x' select='b'/>
    <xsl:value-of select='$x'/>
  </xsl:template>

  <xsl:template match="ENTRY">
    <xsl:variable name='x' select='c'/>
    <doc>
      <xsl:variable name='x' select='d'/>
      <xsl:value-of select='$x'/>
    </doc>
  </xsl:template>

</xsl:stylesheet>
"""


source_2 = """
<dummy/>
"""


expected_1 = """<HTML>
  <HEAD>
    <META HTTP-EQUIV='Content-Type' CONTENT='text/html; charset=iso-8859-1'>
    <TITLE>Address Book</TITLE>
  </HEAD>
  <BODY>
    <TABLE>
      <TR>
        <TD WIDTH='Pieter Aaron'>
          <B>Pieter Aaron</B>
        </TD>
      </TR>
      <TR>
        <TD WIDTH='Emeka Ndubuisi'>
          <B>Emeka Ndubuisi</B>
        </TD>
      </TR>
      <TR>
        <TD WIDTH='Vasia Zhugenev'>
          <B>Vasia Zhugenev</B>
        </TD>
      </TR>
    </TABLE>
  </BODY>
</HTML>"""


expected_2 = """<?xml version='1.0' encoding='UTF-8'?>
<doc>xyz</doc>"""


def Test(tester):


    tester.startGroup("Variable, test 1")

    source = test_harness.FileInfo(uri="addr_book1.xml",nsAware = 0)
    sty = test_harness.FileInfo(string=sheet_str_1)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected_1,
                                    )

    res = xtest.test(tester)
    tester.groupDone()

    if not res:
        return res

    tester.startGroup("Variable, test 2")

    source = test_harness.FileInfo(string=source_2)
    sty = test_harness.FileInfo(string=sheet_str_2)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected_2,
                                    topLevelParams={'override': 'xyz'},
                                    )

    res = xtest.test(tester)
    tester.groupDone()

    tester.startGroup("Variable error, test 1")

    source = test_harness.FileInfo(uri="addr_book1.xml", nsAware=0)
    sty = test_harness.FileInfo(string=invalid_sheet_str_1)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected_1,
                                    expectedException = XsltException,
                                    exceptionValues = {'errorCode':Error.ILLEGAL_SHADOWING},
                                    )
    xtest.test(tester,)
    tester.groupDone()
    return res

if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)

